@@ -80,6 +80,69 @@ def get_client(api_key, api_key_label=None):
     return client
 
 
+def validate_function_name(function_name, verbose=True):
+    """
+    Validate that function name follows MLOps standard: model-service-env
+
+    Input: (string) function name to validate
+    Output: (bool) True if name is valid, False otherwise
+    """
+    supported_services = ["prediction", "training"]
+    supported_environments = ["dev", "test", "prod"]
+    try:
+        model, service, environment = function_name.split("-")
+    except ValueError:
+        if verbose:
+            m = f"Expected function name format: 'model-service-environment'"
+            logging.error(m)
+        return False
+    if service not in supported_services:
+        if verbose:
+            m = f"Supported services: {supported_services}"
+            logging.error(m)
+        return False
+    if environment not in supported_environments:
+        if verbose:
+            m = f"Supported environments: {supported_environments}"
+            logging.error(m)
+        return False
+    return True
+
+
+def delete_function(function_name, confirm=True):
+    """
+    Delete a deployed function 
+
+    Input:
+        - function_name: (string) function name (external id), with the format
+            'model_name-service-environment'. Use `list_functions` to get an
+            overview of deployed functions
+        - confirm: (bool) whether the user will be asked to confirm deletion
+    """
+    if not validate_function_name(function_name):
+        raise ValueError()
+    model, service, environment = function_name.split("-")
+
+    confirmed = False
+
+    if confirm:
+        def confirm_prompt(question):
+            reply = None
+            while reply not in ("y", "n"):
+                reply = input(f"{question} (y/n): ").lower()
+            return (reply == "y")
+        question = f"Delete {model=}, {service=}, {environment=}?"
+        confirmed = confirm_prompt(question)
+
+    if not confirm or confirmed:
+        client = global_client["functions"]
+        try:
+            client.functions.delete(external_id=function_name)
+            logging.debug(f"Deleted function with {function_name=}")
+        except CogniteNotFoundError:
+            logging.error(f"Couldn't find function with '{function_name=}'")
+
+
 def create_function_from_folder(
     function_name, 
     folder, 
@@ -329,6 +392,8 @@ def test_function(function_name, data):
     status is 'ok'
     """
     logging.debug(f"Test function {function_name}")
+    if not validate_function_name(function_name):
+        raise ValueError()
     output = call_function(function_name, data)
     assert output['status'] == 'ok'
     logging.info(f"Test call was successful :)")
@@ -364,31 +429,34 @@ def wait_function_status(function, status=["Ready", "Failed"]):
     return function.status
 
 
-def list_services(tags=[], env='prod'):
+def list_functions(tags=[], env=None):
     """
-    List available services (i.e. CDF Functions) in an environment (dev, test or
-    prod) that match a set of tags.
-    If not tags are provided, all services are listed.
+    List deployed functions, optionally filtering by environment (dev, test 
+    or prod) or set of tags. 
+    
     Input:
         - tags: ([string]): list of tags to search for in the function
-        description
-        - env: (string) the environment, default is 'prod' (recommended)
+        description (or-search)
+        - env: (string) the environment
+    
     Output:
         - ([string]): list of function names (i.e. external_id's )
-    """
-    def get_function_environment(function_name):
-        try:
-            return function_name.rsplit('-', 1)[1]
-        except IndexError:
-            return None
-    
+    """ 
     client = global_client["functions"]
-    functions = [f for f in client.functions.list()
-                    if get_function_environment(f.external_id) == env]
+    functions = client.functions.list(limit=-1)
+    validate_function = (
+        lambda f: validate_function_name(f.external_id, verbose=False)
+    )
+    functions = filter(validate_function, functions)
+    if env:
+        get_function_environment = lambda f: f.external_id.split("-")[2]
+        env_index = lambda f: get_function_environment(f) == env
+        functions = filter(env_index, functions)
     if tags:
-        functions = [f.external_id for f in functions if 
-        any([True if tag in f.description else False for tag in tags])
-        ]
+        contains_tag = lambda f,t: True if t in f.description else False
+        tags_index = lambda f: any([contains_tag(f,t) for t in tags])
+        functions = filter(tags_index, functions)
+    functions = [f.external_id for f in functions]
     return functions
 
 
