@@ -6,6 +6,7 @@ Model registry uses CDF Files.
 """ 
 import os
 import sys
+import json
 import traceback
 import subprocess
 from pathlib import Path
@@ -52,14 +53,15 @@ def deploy_model(model_settings, platform_methods=platform_methods):
         os.chdir(deployment_folder)
         helpers.set_up_requirements(c)
         config.store_service_settings(c)
-        # * It is important to run tests after setting up the artifact
-        #   folder in case it's used to test prediction service.
-        # * Tests need the model requirements installed!
+
         logging.info(f"Run model and service tests")
         if c.test_file:
+            # It should be run as an independent process so that the right
+            # packages are imported, the import paths are correct, etc. 
             command = [sys.executable, "-m", 
                         "akerbp.mlops.services.test_service"]
-            subprocess.check_call(command)
+            output = subprocess.check_output(command, encoding='UTF-8')
+            model_input = json.loads(output.splitlines()[-1])
         else:
             logging.warning("Model test file is missing! Didn't run tests")
 
@@ -67,9 +69,9 @@ def deploy_model(model_settings, platform_methods=platform_methods):
         deploy_function, test_function = platform_methods[c.platform]
         deploy_function(function_name, info=c.info[service_name])
         
-        if c.test_import_path:
+        if c.test_file:
             logging.info("Make a test call")
-            test_function(function_name, input)
+            test_function(function_name, model_input)
         else:
             logging.warning("No test file was set up. End-to-end test skipped!")
     
