@@ -5,7 +5,7 @@ from pathlib import Path
 import akerbp.mlops.cdf.helpers as cdf
 from akerbp.mlops.core import config, logger
 env = config.envs.env
-logging=logger.get_logger(name='MLOps')
+logging=logger.get_logger(name='mlops_model_manager')
 
 
 def setup(cdf_api_keys=None):
